// Copyright (c) 2013, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

import 'dart:html';
import 'dart:async';
import 'package:observatory/models.dart' as M show IsolateRef, ICDataRef;
import 'package:observatory/src/elements/helpers/rendering_scheduler.dart';
import 'package:observatory/src/elements/helpers/tag.dart';
import 'package:observatory/src/elements/helpers/uris.dart';

class ICDataRefElement extends HtmlElement implements Renderable {
  static const tag = const Tag<ICDataRefElement>('icdata-ref');

  RenderingScheduler<ICDataRefElement> _r;

  Stream<RenderedEvent<ICDataRefElement>> get onRendered => _r.onRendered;

  M.IsolateRef _isolate;
  M.ICDataRef _icdata;

  M.IsolateRef get isolate => _isolate;
  M.ICDataRef get icdata => _icdata;

  factory ICDataRefElement(M.IsolateRef isolate, M.ICDataRef icdata,
      {RenderingQueue queue}) {
    assert(isolate != null);
    assert(icdata != null);
    ICDataRefElement e = document.createElement(tag.name);
    e._r = new RenderingScheduler(e, queue: queue);
    e._isolate = isolate;
    e._icdata = icdata;
    return e;
  }

  ICDataRefElement.created() : super.created();

  @override
  void attached() {
    super.attached();
    _r.enable();
  }

  @override
  void detached() {
    super.detached();
    _r.disable(notify: true);
    children = [];
  }

  void render() {
    children = [
      new AnchorElement(href: Uris.inspect(_isolate, object: _icdata))
        ..children = [
          new SpanElement()
            ..classes = ['emphasize']
            ..text = 'ICData',
          new SpanElement()..text = ' (${_icdata.selector})'
        ]
    ];
  }
}
