/*
 * Copyright (c) 2016, the Dart project authors.  Please see the AUTHORS file
 * for details. All rights reserved. Use of this source code is governed by a
 * BSD-style license that can be found in the LICENSE file.
 */
/**
 * @assertion Stream asyncExpand(Stream convert(T event))
 * Creates a new stream with the events of a stream per original event.
 *
 * This acts like expand, except that convert returns a Stream instead of an
 * Iterable. The events of the returned stream becomes the events of the
 * returned stream, in the order they are produced.
 *
 * @description Checks that returned stream produces all the values generated by
 * the convert function.
 * @author ngl@unipro.ru
 */

import "dart:async";
import "../../../Utils/async_utils.dart";
import "../../../Utils/expect.dart";

check(List list, Iterable convert(var value), List expected) {
  Stream s1 = new Stream.fromIterable(list);
  Stream _convert(var event) {
    List lc = new List();
    List l = convert(event);
    for (var e in l) {
      lc.add(e);
    }
    return new Stream.fromIterable(lc);
  }
  Stream s2 = s1.asyncExpand(_convert);
  List actual = new List();
  StreamSubscription ss = s2.listen((event) {actual.add(event);});
  asyncStart();
  ss.onDone(() {
    Expect.listEquals(expected, actual, "expected $expected, actual $actual");
    asyncEnd();
  });
}

void checkList(List list) {
  check(list, (var value) => [], []);
  check(list, (var value) => [value], list);

  List expected = new List();
  for (var el in list) {
    expected.add(el);
    expected.add(el);
  }
  check(list, (var value) => [value, value], expected);

  expected = new List();
  for (var el in list) {
    expected.add(el);
    expected.add(null);
  }
  check(list, (var value) => [value, null], expected);
}

main() {
  checkList([]);
  checkList([null]);
  checkList([1, 2, 3]);
}

